/**
 * Stream representation to include the process (override of the gulp protractor plugin)
 *
 * @author Julien Roche
 * @version 0.4.0
 * @since 0.0.1
 */

'use strict';

// Imports
const gutil = require('gulp-util');
const es = require('event-stream');
const path = require('path');

// Constants
const PluginError = gutil.PluginError;
const PLUGIN_NAME = require('./constants.json').PLUGIN_NAME;

module.exports = function (options, webDriverUrl, autoStartServer, webDriver) {
    let files = [];
    let args = options.args ? options.args.slice(0) : [ ];
    let verbose = options.verbose !== false;

    if (options.debug) {
        args.push('debug');
    }

    return es.through(
        function(file) {
            files.push(file.path);
        },
        function() {
            // Attach Files, if any
            if (files.length) {
                args.push('--specs');
                args.push(files.join(','));
            }

            // Pass in the config file
            let configFilePath = path.resolve(path.join(process.cwd(), options.configFile));
            gutil.log(PLUGIN_NAME + ' - We have the config file to the following path: ' + configFilePath);
            args.unshift(configFilePath);

            // Start the Web Driver server
            try {
                if (autoStartServer) {
                      let callback = () => {
                        gutil.log(PLUGIN_NAME + ' - We will run the Protractor engine');

                        webDriver
                            .runProtractorAndWait(args, (code) => {
                                if (this) {
                                    try {
                                        webDriver.webDriverStandaloneStop(webDriverUrl, () => {
                                            if (code) {
                                                this.emit('error', new PluginError(PLUGIN_NAME, 'protractor exited with code ' + code));

                                            } else {
                                                this.emit('end');
                                            }
                                        });

                                    } catch (err) {
                                        this.emit('error', new PluginError(PLUGIN_NAME, err));
                                    }
                                }
                            });
                    };
                    // Start the update (maybe), run the server, run protractor and stop the server
                    if (options.webDriverUpdate && options.webDriverUpdate.skip) {
                      webDriver.webDriverStandaloneStart(callback, verbose, options.webDriverStart);
                    } else {
                      webDriver.webDriverUpdateAndStart(callback, verbose, options.webDriverUpdate, options.webDriverStart);
                    }
                } else {
                    // Just run protractor
                    webDriver.runProtractorAndWait(args, (code) => {
                        if (this) {
                            if (code) {
                                this.emit('error', new PluginError(PLUGIN_NAME, 'protractor exited with code ' + code));

                            } else {
                                this.emit('end');
                            }
                        }
                    });
                }

            } catch (err) {
                this.emit('error', new PluginError(PLUGIN_NAME, err));
            }
        });
};
